/*_______________________  gp.typedefs.h   _________________________*/

/*       1         2         3         4         5         6        */
/*345678901234567890123456789012345678901234567890123456789012345678*/
/*********************************|**********************************/

/*********************************|**********************************/
/*                                                                  */
/* Copyright (c) 2011-2011 SEIKO EPSON CORP.   All Rights Reserved  */
/*                                                                  */
/*                    Common Public License - v 1.0                 */
/*   THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS   */
/*   COMMON PUBLIC LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR  */
/*   DISTRIBUTION OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE */
/*   OF THIS AGREEMENT.                                             */
/*                                                                  */
/*   The Common Public License in its entirety is located in        */
/*                                                                  */
/*                    gpCommonPublicLicense.txt                     */
/*                                                                  */
/*   and applies to the contents of this file.                      */
/*                                                                  */
/*******%*******************%*****|**********************************/
/*                                                                  */
/*             General Purpose Common Type Definitions              */
/*                                                                  */
/*******%*******************%*****|**********************************/
/*                                                                  */
/* Code States:                                                     */
/*      Initial                                                     */
/*      Developing                                                  */
/*      Working                                                     */
/*      Tested                                                      */
/*      Refactored                                                  */
/*      Final Candidate                                             */
/*                          Final Code                              */
/*                                                                  */
/*******%*******************%*****|**********************************/
#ifndef __GENERAL_PURPOSE_TYPEDEFS_H__
#define __GENERAL_PURPOSE_TYPEDEFS_H__

/*-----------------------  System Includes   -----------------------*/
/*................................|.................................*/
    /*** System                                                     */
    /*** .......................................................... */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <sys/time.h>
#include <time.h>
//#include <dlfcn.h> /* dladdr,dlclose,dlerror,dlopen,dlsym,dlvsym    */
#include <unistd.h>

    /*** Specify the current Graphic System                         */
    /*** .......................................................... */
#define GTKGUI  0
#define QTGUI   1
#define VENDOR_HEADER   "op.vendor.print.dialog.dat.h"

    /*** General Purpose Generic/Common Headers                     */
    /*** .......................................................... */

/*-------------------- Define Basic Data Types ---------------------*/
/*................................|.................................*/
/* gpBIT8 ........ unsigned  8-bit  Min: 0           Max: 255       */
/* gpBIT16 ....... unsigned 16-bit  Min: 0           Max: 65535     */
/* gpBIT32 ....... unsigned 32-bit  Min: 0           Max: 4294967295*/
/* gpCHAR ........ 8-bit char set                                   */
/* gpINT8 ........ signed    8-bit  Min: -128        Max: 127       */
/* gpINT16 ....... signed   16-bit  Min: -32768      Max: 32767     */
/* gpINT32 ....... signed   32-bit  Min:-2147483648  Max; 2147483647*/
/* gpBOOL ........ Boolean type                                     */
/* gpERR_CODE .... Error code for API's and routines                */
/* gpVOID ........ Void pointer                                     */
/* gpINT32  (*FncPtr)(gpVOID*,...) A pointer to a INT32 function    */
/*																																	*/
/* Notes: "size_t" is an unsigned int; gpBIT32 will be used instead.*/
/*................................|.................................*/
typedef unsigned char  gpBIT8;
typedef unsigned short gpBIT16;
typedef unsigned int   gpBIT32;
typedef char           gpCHAR;
typedef char           gpINT8;
typedef short          gpINT16;
typedef int            gpINT32;
typedef gpINT32        gpBOOL;
typedef gpINT32        gpERR_CODE;
typedef void           gpVOID;
typedef gpINT32				(*FncPtr)(gpVOID*,...);

    /*** Definition for Basic Data Type                             */
    /*** .......................................................... */
typedef enum {
    gpTYPE_BIT32 = 0x00,        /* Min: 0           Max: 4294967295 */
    gpTYPE_INT32,               /* Min:-2147483648  Max; 2147483647 */
    gpTYPE_BIT16,               /* Min: 0           Max: 65535			*/
    gpTYPE_INT16,               /* Min: -32768      Max: 32767			*/
    gpTYPE_BIT8,                /* Min: 0           Max: 255				*/
    gpTYPE_INT8,                /* Min: -128        Max: 127				*/
    gpTYPE_CHAR,                /* Text															*/
    gpTYPE_BOOLEAN,             /* TRUE or FALSE										*/
    gpTYPE_FLOAT,
    gpTYPE_DOUBLE,
    gpTYPE_LONG_DOUBLE,
    gpTYPE_1,                   /* Not Used													*/
    gpTYPE_2,                   /* Not Used                         */
    gpTYPE_3,                   /* Not Used                         */
    gpTYPE_24BIT_RGB,           /* 24-BIT RGB Image Data            */
    gpTYPE_ERROR,               /* Min:-2147483648  Max; 0          */

    gpTYPE_VOID    = 0x10,      /* Void type                        */
    gpTYPE_PTR     = 0x20,      /* Type is Pointer                  */
    gpTYPE_FNCPTR  = 0x30,      /* Type is Function Pointer         */
    gpTYPE_PTR_PTR = 0x40,      /* Type is Pointer-to-Pointer       */

    gpTYPE_EOL                  /* End-of-List for Types            */

} gpTYPE;

    /*** Error Callback Functions                                   */
    /*** .......................................................... */
typedef gpVOID    (*CBkFncPtr)(gpINT32); /* Error callback fnc      */

    /*** Memory Functions                                           */
    /*** .......................................................... */
typedef gpVOID* (*epsMemAllocFncPtr)(gpBIT32);          /* Allocate */
typedef gpVOID* (*epsMemReallocFncPtr)(gpVOID*,gpBIT32);/*Reallocate*/
typedef gpVOID  (*epsMemFreeFncPtr)(gpVOID*);           /* Free     */

/*----------------  Data Structure Declarations   ------------------*/
/*................................|.................................*/
    /*** Basic Position Object                                      */
    /*** .......................................................... */
typedef struct _tagEPS_POSITION_ {
    gpINT32 x;
    gpINT32 y;
} gpPOS, *gpPOSID;

    /*** Basic Simple Size Object                                   */
    /*** .......................................................... */
typedef struct _tagEPS_SIZE_ {
    gpINT32 width;
    gpINT32 height;
} gpSIZE, *gpSIZEID;

    /*** Basic Rect Object                                          */
    /*** .......................................................... */
typedef struct _tagEPS_RECT_ {
    gpINT32 top;
    gpINT32 left;
    gpINT32 bottom;
    gpINT32 right;
    gpINT32 width;
    gpINT32 height;
} gpRECT, *gpRTID;

    /*** Generic Image Object                                       */
    /*** .......................................................... */
typedef struct _tagEPS_IMAGE_ {
    gpCHAR*    fileName;          /* Image file name                */
    gpINT32    fileType;          /* Image file type                */
    FILE*       fileFP;           /* File Pointer Struct            */

    gpBIT8*    data;              /* Image/Raster data              */
    gpRECT     rect;              /* Position of image              */
    gpINT32    width;             /* Width  in pixels               */
    gpINT32    height;            /* Height in lines                */
    gpINT32    bpl;               /* Bytes-Per-Line                 */
    gpINT32    size;              /* Total size, in bytes           */
    gpBIT8     bpc;               /* Bits-Per-Color                 */
    gpBIT8     bpp;               /* Bytes-Per-Pixel                */
    gpINT32    colorSpace;        /* Color Space of Image           */
    gpINT32    numColorCh;        /* Num of Color Channels          */
    gpINT32    paletteSize;       /* Size RGB palette               */
    gpBIT8*    paletteData;       /* RGB color palette              */

    gpVOID*    ext;               /* External/Extension             */
} gpIMAGE, *gpIMGID;

/*-------------------- Compiler Configuration ----------------------*/
/*................................|.................................*/


/*---------------------- Basic State Types -------------------------*/
/*................................|.................................*/
#define gpNULL                        0 /* Value of null            */
#define gpTRUE                        1 /* Value of "true"          */
#define gpFALSE                       0 /* Value of "false"         */
#define gpSUCCESSFUL             962384 /* Value of "successful"    */

/*-------------------------- Global Error --------------------------*/
/*................................|.................................*/
#define gpERROR_NONE       gpSUCCESSFUL /* Completed successfully   */
#define gpERROR_OPR_FAIL             -1 /* Generic/Internal Error   */
#define gpERROR_MEMORY_ALLOCATION    -2 /* Memory Allocation Error  */
#define gpERROR_INVALID_ARG          -3 /* Invalid Argument         */
#define gpERROR_INVALID_DATA         -4 /* Invalid Data in Struct   */
#define gpERROR_FATAL             -9999 /* Fatal Error              */

/*---------------------------  Macros   ----------------------------*/
/*................................|.................................*/
#define tolower(c) (c) >= 'A' && (c) <= 'Z' ? (c) + 32 : c

#define Prt(x)       fprintf(stderr,(x)    );fflush(stderr);
#define PrtVal(x,v)  fprintf(stderr,(x),(v));fflush(stderr);

#define GpMin(x,y)  (((x)<(y)) ? (x) : (y)) /* Minimum of x and y   */
#define GpMax(x,y)  (((x)>(y)) ? (x) : (y)) /* Maximum of x and y   */

/* Binary constant generator macro                                  */
/*      By Tom Torfs - donated to the public domain                 */
#define HEX__(n) 0x##n##LU          /* Numeric literal > hex value  */
#define B8__(x)  ((x&0x0000000FLU)?  1:0) \
                +((x&0x000000F0LU)?  2:0) \
                +((x&0x00000F00LU)?  4:0) \
                +((x&0x0000F000LU)?  8:0) \
                +((x&0x000F0000LU)? 16:0) \
                +((x&0x00F00000LU)? 32:0) \
                +((x&0x0F000000LU)? 64:0) \
                +((x&0xF0000000LU)?128:0)

#define B8(d)                    ((gpBIT8 )B8__(HEX__(d)))
#define B16(dmsb,dlsb)          (((gpBIT16)B8(dmsb)<<  8) + B8(dlsb))
#define B32(dmsb,db2,db3,dlsb)  (((gpBIT32)B8(dmsb)<< 24)  \
                               + ((gpBIT32)B8( db2)<< 16)  \
                               + ((gpBIT32)B8( db3)<<  8)  \
                               +            B8(dlsb)          )
/*----------- Create a 32-bit Integer from a 32/64 Address ---------*/
/*................................|.................................*/
#if __x86_64__ || __ppc64__
#define ENV64
#define gpPTR2INT(x)  ((gpBIT32)(0xFFFFFFFF & (long unsigned int)(x)))
#else
#define ENV32
#define gpPTR2INT(x) ((gpBIT32)(x))
#endif
/*------------------------ CPU Endian-ness -------------------------*/
/*................................|.................................*/
typedef enum {
    EPS_ENDIAN_NOT_TESTED = 1000,
    EPS_ENDIAN_BIG,
    EPS_ENDIAN_LITTLE
} gpENDIAN;
typedef enum {
    gp2_BYTES   = 2000,
    gp4_BYTES   = 4000
} gpBYTE_SIZE;

#define gpSwap2B(data) \
        ((((data) >>  8) & 0x00FF    )|(((data) <<  8) & 0xFF00    ) )
#define gpSwap4B(data) \
       ((((data) >> 24) & 0x000000FF)|(((data) >>  8) & 0x0000FF00) |\
        (((data) <<  8) & 0x00FF0000)|(((data) << 24) & 0xFF000000) )

#endif  /* __GENERAL_PURPOSE_TYPEDEFS_H__                           */

/*_______________________  gp.typedefs.h   _________________________*/

/*345678901234567890123456789012345678901234567890123456789012345678*/
/*       1         2         3         4         5         6        */
/*********************************|**********************************/

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*** End of File *** End of File *** End of File *** End of File ****/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%|%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
